# Detection of Potential Geothermal Exploration Sites from Hyperspectral Images via Deep Learning
# Author: Ebubekir Demir <edemir@mines.edu>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from sklearn.metrics import roc_curve, auc
from osgeo import gdal_array
import os


def getdata(folder):
    data = []
    for filename in os.listdir(folder):
        fp = os.path.join(folder, filename)
        df = pd.read_csv(fp)
        x = df["X"].iloc[0]
        y = df["Y"].iloc[0]
        res = []
        res.append(x)
        res.append(y)
        data.append(res)
        # print(x, y)
    dataa = np.array(pd.DataFrame(np.array(data), columns=['X', 'Y']).sort_values(by=['X']))
    print(folder, dataa)
    return dataa


def roc(list, savefile, title):
    fig, axs = plt.subplots(1, 3, layout='constrained')
    for idx, ls in enumerate(list):
        pltnum = idx // 3
        print(ls[0])
        gnd_i = gdal_array.LoadFile(ls[0])
        fusion_i = gdal_array.LoadFile(ls[1])
        if fusion_i.dtype == np.dtype('complex128'):
            fusion_i = fusion_i.astype('float128')
        if fusion_i.dtype == np.dtype('complex64'):
            fusion_i = fusion_i.astype('float64')
        gnd = gnd_i[~np.isnan(gnd_i) & ~np.isnan(fusion_i)]
        fusion = fusion_i[~np.isnan(gnd_i) & ~np.isnan(fusion_i)]
        gnd[gnd >= 0.5] = 1
        gnd[gnd <= 0.5] = 0
        fpr, tpr, _ = roc_curve(gnd, fusion)
        roc_auc = auc(fpr, tpr)
        lw = 2
        axs[pltnum].plot(
            fpr,
            tpr,
            color=ls[2],
            lw=lw,
            label=(ls[3] + " (auc=%0.3f)") % roc_auc,
        )

        axs[pltnum].plot([0, 1], [0, 1], color="navy", lw=lw, linestyle="--")
        # axs[pltnum].set_xlim([0.0, 1.05])
        # axs[pltnum].set_ylim([0.0, 1.05])
        axs[pltnum].set_xlabel("False Positive Rate")
        axs[pltnum].set_ylabel("True Positive Rate")
        axs[pltnum].set_title(title)
        # axs[pltnum].legend(loc="lower right")
    plt.savefig(savefile, dpi=300)
    plt.show()


if __name__ == '__main__':
    roc(ls, sf, title)
