# Detection of Potential Geothermal Exploration Sites from Hyperspectral Images via Deep Learning
# Author: Ebubekir Demir <edemir@mines.edu>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

from osgeo import gdal, ogr, osr
from tqdm import tqdm

import numpy as np
import pandas as pd
import math
import argparse
import os


def fusion(opal, hema, alun, kaol, chal, fus):
    driver = gdal.GetDriverByName('GTiff')
    ds = gdal.Open(opal)
    gt = ds.GetGeoTransform()
    crs = ds.GetProjection()
    cols = ds.RasterXSize
    rows = ds.RasterYSize

    d_opal = ds.GetRasterBand(1).ReadAsArray()
    d_hema_f = gdal.Open(hema)
    d_hema = d_hema_f.GetRasterBand(1).ReadAsArray()
    d_alun_f = gdal.Open(alun)
    d_alun = d_alun_f.GetRasterBand(1).ReadAsArray()
    d_kaol_f = gdal.Open(kaol)
    d_kaol = d_kaol_f.GetRasterBand(1).ReadAsArray()
    d_chal_f = gdal.Open(chal)
    d_chal = d_chal_f.GetRasterBand(1).ReadAsArray()
    d_opal = np.nan_to_num(d_opal)
    d_hema = np.nan_to_num(d_hema)
    d_alun = np.nan_to_num(d_alun)
    d_kaol = np.nan_to_num(d_kaol)
    d_chal = np.nan_to_num(d_chal)

    dt = np.stack((d_opal, d_chal, d_hema, d_kaol, d_alun))
    data = dt.sum(axis=0)
    print(data.min())
    print(data.max())

    target_ds = driver.Create(fus, cols, rows, 1, gdal.GDT_CFloat32)
    target_ds.SetGeoTransform(gt)
    target_ds.SetProjection(crs)
    band = target_ds.GetRasterBand(1)
    band.WriteArray(data, 0, 0)
    band.FlushCache()
    res = None

if __name__ == '__main__':
    opal = "opal path"
    alun = "alun path"
    hema = "hema path"
    kaol = "kaol path"
    chal = "chal path"
    fus = "fus path"
    fusion(opal, hema, alun, kaol, chal, fus)
