# Detection of Potential Geothermal Exploration Sites from Hyperspectral Images via Deep Learning
# Author: Ebubekir Demir <edemir@mines.edu>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.


import pandas as pd
import os
from sklearn.cluster  import MiniBatchKMeans
from sklearn.cluster  import KMeans
import numpy as np
from sklearn.model_selection import KFold
from tqdm import tqdm
import seaborn as sns
import matplotlib.pyplot as plt
import math

def run(data, split):
    kf = KFold(n_splits=split, shuffle=True)
    data = pd.DataFrame(data)
    result_dist = []
    result_iner = []
 
    for cluster_count in tqdm(range(1, 30)): #bu kmeansteki k
        kmeans_data= np.array(data)
        kmeans_data= kmeans_data.reshape(-1, 1)
 
        km_tr = KMeans(n_clusters=cluster_count, max_iter=50).fit(kmeans_data)
        result_iner.append(km_tr.inertia_)
        res123 = []

        for train_index, test_index in kf.split(data):
 
            #print("TRAIN:", train_index, "TEST:", test_index)
            X_train, X_test = data.iloc[train_index], data.iloc[test_index]
 
            disc_tr = X_train
            disc_tr = np.array(disc_tr)
            disc_tr = disc_tr.reshape(-1, 1)
 
            disc_te = X_test
            disc_te = np.array(disc_te)
            disc_te = disc_te.reshape(-1, 1)
 
            km_tr = KMeans(n_clusters=cluster_count, max_iter=500).fit(disc_tr)
            km_te = KMeans(n_clusters=cluster_count, max_iter=500).fit(disc_te)
 
            cl_ctr_tr = km_tr.cluster_centers_
            cl_ctr_te = km_te.cluster_centers_
 
            cl_ctr_tr.transpose().sort()
            cl_ctr_te.transpose().sort()
 
            diff = cl_ctr_tr - cl_ctr_te
            res = sum(diff ** 2)
            res123.append(res)
 
        res234 = sum(res123)
        result_dist.append(res234/cluster_count)
        

    print(result_dist)
    print(result_iner)

def elb_run(data):
    data = pd.DataFrame(data)
    result_iner = []
 
    for cluster_count in tqdm(range(1, 30)): #bu kmeansteki k
        kmeans_data= np.array(data)
        kmeans_data= kmeans_data.reshape(-1, 1)
 
        km_tr = KMeans(n_clusters=cluster_count, max_iter=500).fit(kmeans_data)
        result_iner.append(km_tr.inertia_)

    res, lst = elbow(result_iner)
    print("k:", res)
    return res, lst

    
def elbow(data):
    arr = np.array(data)
    max = arr.max()
    min = arr.min()
    lenght = arr.size
    ratio = (max - min) / lenght
    dt = arr / ratio
    mx = 180
    result = 0
    ls = []
    for i in range(1, lenght-1):
        ang = getangle(i-1, dt[i-1], i, dt[i], i+1, dt[i+1])
        ls.append(round(ang,2))
        if ang <= mx:
            mx = ang
            result = i+1
    return result, ls

def getangle(ax, ay, bx, by, cx ,cy):
    a2 = (bx - ax)*(bx - ax) + (by - ay)*(by - ay)
    b2 = (cx - ax)*(cx - ax) + (cy - ay)*(cy - ay)
    c2 = (cx - bx)*(cx - bx) + (cy - by)*(cy - by)
    cosx = (a2 + c2 - b2) / (2.0 * math.sqrt(a2*c2))
    try:
        res = math.acos(cosx) * (180.0 / math.pi)
    except:
        res = 200
        print("ERROR GETANGLE", cosx, ax, ay, bx, by, cx ,cy, a2, b2, c2)
    return res
